@@ -48,7 +48,7 @@ class DataParser:
                 if item["is_dir"]:
                     child = Collection(path, modified, icon, has_thumb, parent_root)
                 else:
-                    child = Resource(path, size, modified, item["mime_type"], icon, has_thumb, parent_root)
+                    child = Resource(path, size, item["bytes"], modified, item["mime_type"], icon, has_thumb, parent_root)
                     if path.startswith("/Public/"):
                         child.generate_public_link(self.uid)
                 folder.add_item(child)
@@ -59,16 +59,8 @@ class DataParser:
                 self.tree_controller.set_root_folder(folder)
             else:
                 self.tree_controller.update_folder(folder.path, folder)
-
             tree_item = folder
 
-##            # Expand previously opened folders
-##            if opened_folders != None:
-##                for child_folder in folder.get_folders():
-##                    if opened_folders.has_key(child_folder.get_name()):
-##                        if child_folder.path == opened_folders[child_folder.get_name()]:
-##                            self.controller.connection.get_metadata(child_folder.path, child_folder.root)
-
         return tree_item
 
     def check_deleted(self, data, item, tree_item = None):
@@ -122,24 +114,19 @@ class DataParser:
 
 class MaemoDataHandler:
 
-    def __init__(self, controller, maemo, logger):
+    def __init__(self, controller, logger):
         self.controller = controller
-        self.maemo = maemo
         self.logger = logger
         self.store_auth_to_file = True
         self.dont_show_dl_dialog = False
         self.only_sync_on_wlan = True
-        if self.maemo:
-            self.user_home = str(QDir.home().absolutePath())
-            self.app_root = "/opt/dropn900/"
-            self.config_root = self.user_home + "/.dropn900/"
-            self.data_root = self.user_home + "/MyDocs/DropN900/"
-            self.default_data_root = self.user_home + "/MyDocs/DropN900"
-        else:
-            self.app_root = ""
-            self.config_root = ""
-            self.data_root = ""
-            self.default_data_root = ""
+
+        self.user_home = str(QDir.home().absolutePath())
+        self.app_root = "/opt/dropn900/"
+        self.config_root = self.user_home + "/.dropn900/"
+        self.data_root = self.user_home + "/MyDocs/DropN900/"
+        self.default_data_root = self.user_home + "/MyDocs/DropN900"
+
         self.startup_checks()
 
     def datapath(self, datafile):
@@ -159,8 +146,6 @@ class MaemoDataHandler:
         self.check_for_config_folder()
 
     def check_for_data_folder(self):
-        if not self.maemo:
-            return
         if self.data_root == (self.user_home + "/MyDocs/DropN900/"):
             data_dir = QDir.home()
             if data_dir.cd("MyDocs"):
@@ -181,8 +166,6 @@ class MaemoDataHandler:
                 self.check_for_data_folder()
 
     def check_for_config_folder(self):
-        if not self.maemo:
-            return
         config_dir = QDir.home()
         if not config_dir.cd(".dropn900"):
             if config_dir.mkdir(".dropn900"):
@@ -254,6 +237,7 @@ class Item:
         self.public_link = None
         self.hash = None
         self.size = None
+        self.size_bytes = 0
 
         self.format_parent()
 
@@ -408,12 +392,13 @@ class Collection(Item):
 
 class Resource(Item):
 
-    def __init__(self, path, size, modified, mime_type, icon, has_thumb, root):
+    def __init__(self, path, size, size_bytes, modified, mime_type, icon, has_thumb, root):
         # Init parent class
         Item.__init__(self, path, root, modified, icon, has_thumb)
 
         # Init Resource params
         self.size = size
+        self.size_bytes = size_bytes
         self.mime_type = mime_type
         
         self.set_name()
