@@ -1,8 +1,9 @@
 
 import os
 
-from PyQt4 import QtCore, QtGui
-from PyQt4.QtCore import Qt, QStringList, QTimer
+from PyQt4 import QtCore, QtGui, QtMaemo5
+from PyQt4.QtMaemo5 import QMaemo5InformationBox
+from PyQt4.QtCore import Qt, QString, QStringList, QTimer, QDir
 from PyQt4.QtGui import QMainWindow, QWidget, QTreeWidgetItem, QDesktopServices
 from PyQt4.QtGui import QImage, QPixmap, QIcon, QImageReader, QMovie
 from PyQt4.QtGui import QInputDialog, QFileDialog, QMessageBox
@@ -16,12 +17,6 @@ from ui.ui_managerwidget import Ui_ManagerWidget
 from ui.ui_consolewidget import Ui_ConsoleWidget
 from ui.ui_loadingwidget import Ui_LoadingWidget
 
-try:
-    from PyQt4 import QtMaemo5
-    qt4_maemo5_present = True
-except ImportError:
-    qt4_maemo5_present = False
-
 
 """ UiController controls ui view switching and ui interactions """
 
@@ -35,8 +30,7 @@ class UiController:
 
         ### Main Window
         self.main_widget = QMainWindow()
-        if qt4_maemo5_present == True:
-            self.main_widget.setAttribute(Qt.WA_Maemo5StackedWindow)
+        self.main_widget.setAttribute(Qt.WA_Maemo5StackedWindow)
         self.main_ui = Ui_DropN900Widget()
         self.main_ui.setupUi(self.main_widget)
 
@@ -45,21 +39,26 @@ class UiController:
         self.main_ui.centralwidget.setLayout(self.stacked_layout)
 
         # Menu items
-        self.action_console = QtGui.QAction("Show Console", self.main_widget)
-        self.action_reset_auth = QtGui.QAction("Reset Auth", self.main_widget)
+        self.action_transfers = QtGui.QAction("Transfers", self.main_widget)
+        self.action_settings = QtGui.QAction("Settings", self.main_widget)
+        self.action_sync = QtGui.QAction("Synchronize", self.main_widget)
+        self.action_console = QtGui.QAction("Show Log", self.main_widget)
+        self.action_about = QtGui.QAction("About", self.main_widget)
         self.action_exit = QtGui.QAction("Exit", self.main_widget)
 
+        self.main_ui.menubar.addAction(self.action_transfers)
+        self.main_ui.menubar.addAction(self.action_settings)
+        self.main_ui.menubar.addAction(self.action_sync)
         self.main_ui.menubar.addAction(self.action_console)
-        self.main_ui.menubar.addAction(self.action_reset_auth)
-        if qt4_maemo5_present == True:
-            action_about = QtGui.QAction("About", self.main_widget)
-            self.main_ui.menubar.addAction(action_about)
-            action_about.triggered.connect(self.show_about)
+        self.main_ui.menubar.addAction(self.action_about)
         self.main_ui.menubar.addAction(self.action_exit)
 
         # Connects
+        self.action_transfers.triggered.connect(self.show_transfer_widget)
+        self.action_sync.triggered.connect(self.synchronize_now)
+        self.action_settings.triggered.connect(self.show_settings_widget)
         self.action_console.triggered.connect(self.show_console)
-        self.action_reset_auth.triggered.connect(self.reset_auth)
+        self.action_about.triggered.connect(self.show_about)
         self.action_exit.triggered.connect(self.shut_down)
 
         ### Browser Widget
@@ -91,7 +90,7 @@ class UiController:
         self.trusted_login_widget = QWidget()
         self.trusted_login_ui = Ui_TrustedLoginWidget()
         self.trusted_login_ui.setupUi(self.trusted_login_widget)
-        self.trusted_login_ui.label_icon.setPixmap(QPixmap(self.datahandler.datapath("ui/images/synced_with_dropbox_logo.png")))
+        self.trusted_login_ui.label_icon.setPixmap(QPixmap(self.datahandler.datapath("ui/images/dropn900_logo.png")).scaled(65,65))
 
         # Connects
         self.trusted_login_ui.button_auth.clicked.connect(self.try_trusted_login)
@@ -105,29 +104,32 @@ class UiController:
         tree = self.manager_ui.tree_widget
         self.tree_controller = TreeController(tree, self.manager_ui, self.controller, self.logger)
 
-        # Hide public link elements (not used yet)
-        self.manager_ui.public_link_line_edit.hide()
+        # Hide public link elements on start
+        self.manager_ui.button_copy_public_link.hide()
         self.manager_ui.button_open_public_link.hide()
         
         # Connects
+        self.manager_ui.button_copy_public_link.clicked.connect(self.copy_item_link)
         self.manager_ui.button_open_public_link.clicked.connect(self.open_item_link)
         self.manager_ui.button_download.clicked.connect(self.item_download)
         self.manager_ui.button_upload.clicked.connect(self.item_upload)
         self.manager_ui.button_rename.clicked.connect(self.item_rename)
         self.manager_ui.button_remove.clicked.connect(self.item_remove)
         self.manager_ui.button_new_folder.clicked.connect(self.item_new_folder)
+        self.last_dl_location = None
+        self.last_ul_location = None
 
         ### Console widget
         
-        if qt4_maemo5_present == True:
-            self.console_widget = QWidget(self.main_widget, Qt.Window)
-            self.console_widget.setAttribute(Qt.WA_Maemo5StackedWindow)
-        else:
-            self.console_widget = QWidget()
+        self.console_widget = QWidget(self.main_widget, Qt.Window)
+        self.console_widget.setAttribute(Qt.WA_Maemo5StackedWindow)
         self.console_ui = Ui_ConsoleWidget()
         self.console_ui.setupUi(self.console_widget)
         self.console_ui.button_save.clicked.connect(self.save_log_to_file)
         self.console_ui.button_back.clicked.connect(self.hide_console)
+        
+        ### Settings widget
+        self.settings_widget = None
 
         ### Fill stacked layout
         self.stacked_layout.addWidget(self.browser_widget)
@@ -142,6 +144,8 @@ class UiController:
         self.loading_ui.setupUi(self.loading_widget)
         self.manager_ui.action_layout.insertWidget(3, self.loading_widget)
         self.loading_widget.hide()
+        
+        self.tree_controller.set_loading_ui(self.loading_ui)
 
         # Init loading animation
         self.loading_animation = QMovie(self.datahandler.datapath("ui/images/loading.gif"), "GIF", self.loading_ui.load_animation_label)
@@ -156,23 +160,43 @@ class UiController:
         self.information_message_timer.timeout.connect(self.hide_information_message)
         self.information_icon_ok = QPixmap(self.datahandler.datapath("ui/icons/check.png")).scaled(24,24)
         self.information_icon_error = QPixmap(self.datahandler.datapath("ui/icons/cancel.png")).scaled(24,24)
+        self.information_icon_queue = QPixmap(self.datahandler.datapath("ui/icons/queue.png")).scaled(24,24)
 
+    def set_settings_widget(self, settings_widget):
+        self.settings_widget = settings_widget
+        
+    def set_transfer_widget(self, transfer_widget):
+        self.transfer_widget = transfer_widget
+        
+    def synchronize_now(self):
+        self.settings_widget.sync_now_clicked()
+        
     def show(self):
         # Nokia N900 screen resolution, full screen
         self.main_widget.resize(800, 480) 
         self.main_widget.show()
-        if qt4_maemo5_present:
-            self.main_widget.showMaximized()
+        self.main_widget.showMaximized()
+
+    def show_settings_widget(self):
+        if self.settings_widget != None:
+            if not self.settings_widget.isVisible():
+                self.settings_widget.show()
+                self.settings_widget.check_settings()
+
+    def show_transfer_widget(self):
+        if self.settings_widget.isVisible():
+            self.settings_widget.hide()
+        self.transfer_widget.show()
         
     def show_about(self):
-        if not qt4_maemo5_present:
-            return
-        QtMaemo5.QMaemo5InformationBox.information(None, "Created by Jonne Nauha <jonne.nauha@evocativi.com>\nIssue tracker: http://github.com/jonnenauha/DropN900/issues", 0)
+        message = "<span style=\"color:white;\"><span style=\"color:#0099FF;\"><b>Maintainer</b></span> Jonne Nauha - jonne.nauha@evocativi.com<br><span style=\"color:#0099FF;\"><b>Bug Tracker</b></span> http://github.com/jonnenauha/DropN900/issues<br><span style=\"color:#0099FF;\"><b>Icons</b></span> http://openiconlibrary.sourceforge.net</span></span>"
+        QMaemo5InformationBox.information(self.manager_widget, QString(message), 0)
         
-    def show_banner(self, message):
-        if not qt4_maemo5_present:
-            return
-        QtMaemo5.QMaemo5InformationBox.information(None, QtCore.QString(message), 0)
+    def show_note(self, message):
+        QMaemo5InformationBox.information(None, QString(message), 0)
+        
+    def show_banner(self, message, timeout = 5000):
+        QMaemo5InformationBox.information(None, QString(message), timeout)
         
     def show_loading_ui(self, message = "", loading = True):
         self.loading_ui.info_label.setText(message)
@@ -187,20 +211,22 @@ class UiController:
         else:
             self.loading_animation.stop()
 
-    def show_information_ui(self, message, succesfull):
+    def show_information_ui(self, message, succesfull, timeout = 4000):
         self.loading_ui.load_animation_label.hide()
-        if self.manager_ui.thumb_container.isVisible():
-            self.thumb_was_visible = True
+        self.thumb_was_visible = self.manager_ui.thumb_container.isVisible()
+        if self.thumb_was_visible:
             self.manager_ui.thumb_container.hide()
         self.loading_ui.info_label.setText(message)
-        if succesfull:
+        if succesfull == None:
+            icon = self.information_icon_queue
+        elif succesfull:
             icon = self.information_icon_ok
         else:
             icon = self.information_icon_error
         self.loading_ui.info_label_icon.setPixmap(icon)
         self.loading_ui.info_label_icon.show()
         self.loading_widget.show()
-        self.information_message_timer.start(4000)
+        self.information_message_timer.start(timeout)
 
     def hide_loading_ui(self):
         if not self.information_message_timer.isActive():
@@ -242,22 +268,27 @@ class UiController:
             error = "Password can't be empty!"
             
         if error == None:
-            self.trusted_login_ui.label_error.setText("Authenticating, please wait...")
+            self.show_banner("Authenticating...", 3000)
+            self.set_trusted_login_info("Authenticating, please wait...")
             self.truested_email = str(email)
             self.trusted_password = str(password)
             QTimer.singleShot(100, self.do_trusted_login_networking)
         else:
-            self.trusted_login_ui.label_error.setText(error)
-  
+            self.set_trusted_login_error(error)
+            
+    def set_trusted_login_error(self, error):
+        self.trusted_login_ui.label_error.setStyleSheet("color: #9d1414;")
+        self.trusted_login_ui.label_error.setText(error)
+        
+    def set_trusted_login_info(self, info):
+        self.trusted_login_ui.label_error.setStyleSheet("color: #149d2b;")
+        self.trusted_login_ui.label_error.setText(info)
+        
     def do_trusted_login_networking(self):
         self.controller.end_trusted_auth(self.truested_email, self.trusted_password)
         self.truested_email = None
         self.trusted_password = None
         
-    def reset_auth(self):
-        self.reset_trusted_login()
-        self.controller.datahandler.reset_auth()
-        
     def reset_trusted_login(self):
         self.trusted_login_ui.line_edit_email.clear()
         self.trusted_login_ui.line_edit_password.clear()
@@ -287,7 +318,6 @@ class UiController:
     # Signal handlers
             
     def shut_down(self):
-        self.logger.info("Shutting down...")
         self.main_widget.close()
 
     def show_console(self):
@@ -295,7 +325,10 @@ class UiController:
             self.switch_context("console")
         else:
             self.hide_console()
-            
+
+    def hide_console(self):
+        self.console_widget.hide()
+                    
     def save_log_to_file(self):
         filename = self.datahandler.get_data_dir_path() + "dropn900.log"
         log_string = str(self.console_ui.text_area.toPlainText())
@@ -307,9 +340,6 @@ class UiController:
         except IOError:
             self.logger.error("Could not open " + filename + " to save log")
 
-    def hide_console(self):
-        self.console_widget.hide()
-
     # We should not end here in the device as it has ssl libs in place
     # on windows youll have to have openssl libs in place or login via webauth wont work
     def ssl_errors_occurred(self, reply, errors):
@@ -350,27 +380,40 @@ class UiController:
         else:
             self.controller.end_auth()
             
+    def copy_item_link(self):
+        url = self.tree_controller.current_public_link
+        if url == None:
+            return
+        self.datahandler.copy_url_to_clipboard(url)
+    
     def open_item_link(self):
-        url = self.manager_ui.public_link_line_edit.text()
-        self.webview_load_url(url)
-        self.browser_ui.label_info.hide()
-        self.browser_ui.button_done.setText("Back")
-        self.browser_ui.button_done.setIcon(QIcon(self.datahandler.datapath("ui/icons/back.png")))
-        self.switch_context("browser")
+        url = self.tree_controller.current_public_link
+        if url == None:
+            return
+        QDesktopServices.openUrl(QtCore.QUrl(url))
 
     def item_download(self):
         data = self.get_selected_data()
         if data == None:
             return
-
-        # This dialog shows sometimes strange stuff on maemo5
-        # It's a PyQt4 bug and its the best we got, you can cope with this
-        local_folder_path = QFileDialog.getExistingDirectory(self.manager_widget, "Select Download Folder", self.datahandler.get_data_dir_path(), QFileDialog.ShowDirsOnly|QFileDialog.HideNameFilterDetails|QFileDialog.ReadOnly)
-        if local_folder_path.isEmpty():
-            return
-
-        store_path = local_folder_path + "/" + data.name
-        self.controller.connection.get_file(data.path, data.root, str(store_path))
+            
+        if self.datahandler.dont_show_dl_dialog == False:
+            # This dialog shows sometimes strange stuff on maemo5
+            # It's a PyQt4 bug and its the best we got, you can cope with this
+            if self.last_dl_location == None:
+                self.last_dl_location = self.datahandler.get_data_dir_path()
+            local_folder_path = QFileDialog.getExistingDirectory(self.manager_widget, QString("Select Download Folder"), QString(self.last_dl_location), QFileDialog.ShowDirsOnly|QFileDialog.HideNameFilterDetails|QFileDialog.ReadOnly)
+            if local_folder_path.isEmpty():
+                return
+            self.last_dl_location = str(local_folder_path)
+            store_path = local_folder_path + "/" + data.name
+        else:
+            dir_check = QDir(self.datahandler.get_data_dir_path())
+            if not dir_check.exists():
+                self.show_note("Cannot download, destination " + self.datahandler.get_data_dir_path() + " does not exist. Please set a new folder in settings.")
+                return
+            store_path = self.datahandler.get_data_dir_path() + data.name
+        self.controller.connection.get_file(data.path, data.root, str(store_path), data.get_size(), data.mime_type)
         
     def item_upload(self):
         data = self.get_selected_data()
@@ -379,10 +422,13 @@ class UiController:
 
         # This dialog shows sometimes strange stuff on maemo5
         # It's a PyQt4 bug and its the best we got, you can cope with this
-        local_file_path = QFileDialog.getOpenFileName(self.manager_widget, "Select File for Upload", "/home/user")
+        if self.last_ul_location == None:
+            self.last_ul_location = self.datahandler.get_data_dir_path()
+        local_file_path = QFileDialog.getOpenFileName(self.manager_widget, QString("Select File for Upload"), QString(self.last_ul_location))
         if local_file_path.isEmpty():
             return
-
+        parse_path = str(local_file_path)
+        self.last_ul_location = parse_path[0:parse_path.rfind("/")]
         self.controller.connection.upload_file(data.path, data.root, str(local_file_path))
         
     def item_rename(self):
@@ -479,7 +525,9 @@ class TreeController:
         self.last_clicked = None
         self.connection = None
         self.selected_data = None
-
+        self.current_public_link = None
+        self.loading_ui = None
+        
         # Treeview header init
         headers = self.tree.headerItem()
         headers.setTextAlignment(0, Qt.AlignLeft|Qt.AlignVCenter)
@@ -505,14 +553,19 @@ class TreeController:
         # Connects
         self.tree.itemSelectionChanged.connect(self.item_selection_changed)
         self.tree.itemClicked.connect(self.item_clicked)
-        self.tree.itemDoubleClicked.connect(self.item_double_clicked)
+        #self.tree.itemDoubleClicked.connect(self.item_double_clicked)
         self.tree.itemExpanded.connect(self.item_expanded)
         self.tree.itemCollapsed.connect(self.item_collapsed)
 
     def setup(self, connection):
         self.connection = connection
+        
+    def set_loading_ui(self, loading_ui):
+        self.loading_ui = loading_ui
 
     def item_expanded(self, tree_item):
+        self.tree.setCurrentItem(tree_item)
+        self.folder_opened(tree_item)
         self.set_icon("folder_opened", tree_item)
 
     def item_collapsed(self, tree_item):
@@ -533,7 +586,6 @@ class TreeController:
         root_item = QTreeWidgetItem(columns)
 
         self.set_icon(root_folder.mime_type, root_item)
-        self.update_link_area()
         
         self.tree.addTopLevelItem(root_item)
         root_item.setExpanded(True)
@@ -749,6 +801,8 @@ class TreeController:
     def update_thumbnail(self, show, qpixmap = None):
         if qpixmap:
             self.controls_ui.thumb_container.setPixmap(qpixmap)
+        if self.loading_ui.info_label.isVisible():
+            show = False
         self.controls_ui.thumb_container.setVisible(show)
 
     # Qt signal handlers below
@@ -795,11 +849,8 @@ class TreeController:
         # Set selected data
         self.selected_data = data
 
-        # Enable public links once API gets
-        #if not data.is_folder():
-        #    self.update_link_area(data.public_link)
-        #else:
-        #    self.update_link_area()
+        # Show public link if present
+        self.update_link_area(data.public_link)
 
     def set_thumb(self, image_path, root, size = "large"):
         # Check if we already have the thumb
@@ -827,16 +878,18 @@ class TreeController:
         except ValueError:
             return False
 
-    def update_link_area(self, url = None):
-        show = False
-        if url != None:
-            show = True
-            self.controls_ui.public_link_line_edit.setText(url)
-        self.controls_ui.public_link_line_edit.setVisible(show)
-        self.controls_ui.button_open_public_link.setVisible(show)
+    def update_link_area(self, public_link_url):
+        show_link_controls = False
+        if public_link_url != None:
+            show_link_controls = True
+        self.controls_ui.button_copy_public_link.setVisible(show_link_controls)
+        self.controls_ui.button_open_public_link.setVisible(show_link_controls)
+        self.controls_ui.button_upload.setVisible(not show_link_controls)
+        self.controls_ui.button_new_folder.setVisible(not show_link_controls)
+        self.current_public_link = public_link_url
         
-    def item_double_clicked(self, item, column):
-        if item.isExpanded():
+    def folder_opened(self, item):
+        if not item.isExpanded():
             return
         # We always want to reset found tree items when we
         # are building the tree up with metadata fetches
